/*
 * Copyright (c) 1999-2024 Hartmut Seichter
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
#ifndef PW_CORE_MESH_HPP
#define PW_CORE_MESH_HPP

#include <pw/core/globals.hpp>
#include <pw/core/primitives.hpp>

#include <atomic>
#include <variant>

namespace pw {

struct attribute final {
    using mask_type    = std::vector<bool>;
    using int8_type    = std::vector<int8_t>;
    using int32_type   = std::vector<int32_t>;
    using float_type   = std::vector<float>;
    using vector2_type = std::vector<vector2<float>>;
    using vector3_type = std::vector<vector3<float>>;
    using vector4_type = std::vector<vector4<float>>;

    using attribute_data =
        std::variant<std::monostate, mask_type, int8_type, int32_type,
                     float_type, vector2_type, vector3_type, vector4_type>;

    enum attribute_type {
        normals,
        texture_coordinates,
    };

    attribute_type type{};
    attribute_data data{};
};

struct mesh {
    primitives geometry{};
    std::vector<attribute> attributes;

    std::atomic<uint64_t> change_count{0};
};

} // namespace pw

#endif
