/*
 * Copyright (c) 1999-2024 Hartmut Seichter
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
#ifndef PW_CORE_RECTANGLE_HPP
#define PW_CORE_RECTANGLE_HPP

#include <pw/core/point.hpp>
#include <pw/core/size.hpp>

namespace pw {

template <typename Scalar> struct rectangle final {

    point<Scalar> position{};
    size<Scalar> size{};

    constexpr bool contains(const point<Scalar>& p) const noexcept {
        return p.x >= position.x && p.x <= position.x + size.width &&
               p.y >= position.y && p.y <= position.y + size.height;
    }

    template <typename ScalarOut>
    constexpr auto cast() const noexcept -> rectangle<ScalarOut> {
        return {.position = position.template cast<ScalarOut>(),
                .size     = size.template cast<ScalarOut>()};
    }
};

} // namespace pw

#endif
