/*
 * Copyright (c) 1999-2024 Hartmut Seichter
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
#ifndef PW_CORE_SIZE_HPP
#define PW_CORE_SIZE_HPP

#include <pw/core/globals.hpp>
#include <type_traits>

namespace pw {

template <typename Scalar> struct size {

    using value_type = Scalar;
    Scalar width{}, height{};

    constexpr auto area() const noexcept -> Scalar {
        if constexpr (std::is_unsigned_v<Scalar>) {
            return width * height;
        } else {
            return std::abs(width * height);
        }
    }

    template <typename ScalarOut>
    constexpr auto cast(this auto&& self) noexcept -> size<ScalarOut> {
        return {.width  = ScalarOut(self.width),
                .height = ScalarOut(self.height)};
    }

    template <typename Other>
    constexpr auto operator()(this auto&& self) noexcept {
        return Other{self.width, self.height};
    }
};

//
// deduction guide
//
template <class... U, class CT = std::common_type_t<U...>>
size(U...) -> size<CT>;

} // namespace pw

#endif
