/*
 * Copyright (c) 1999-2024 Hartmut Seichter
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 *
 */
#ifndef PW_CORE_TIMER_HPP_
#define PW_CORE_TIMER_HPP_

#include <pw/core/globals.hpp>

#include <chrono>

namespace pw {

/**
 * @brief A simple high resolution timer
 */
struct time final {

    using tick_t = std::chrono::time_point<std::chrono::high_resolution_clock>;

    time()            = default;
    time(const time&) = default;
    ~time()           = default; /// d'tor

    /**
     * @brief reset timer to current system time
     */
    void reset();

    /**
     * @brief elapsed time based on the scale
     * @return value of the time since last @see reset
     */
    double elapsed() const;

    /**
     * @brief time from start of core in seconds
     * @return time in seconds
     */
    static double now();

  protected:
    tick_t _start{std::chrono::high_resolution_clock::now()};
};

} // namespace pw

#endif
