#include <pw/core/aabb.hpp>
#include <pw/core/axisangle.hpp>
#include <pw/core/formatter.hpp>
#include <pw/core/matrix_transform.hpp>
#include <pw/core/mesh.hpp>
#include <pw/core/primitives.hpp>
#include <pw/core/serialize.hpp>
#include <pw/core/vector.hpp>

#include <print>
#include <variant>
#include <vector>

namespace pw {} // namespace pw

auto main() -> int {

    auto geom = pw::primitives{
        .vertices = {{1.2f, 2.4f, 4.8f},
                     {2.4f, 1.2f, 4.8f},
                     {1.2f, 4.8f, 2.4f}},
        .indices  = {0, 1, 2},

        .topology = pw::primitives::topology_type::triangle_list};

    auto mesh = pw::mesh{
        .geometry   = geom,
        .attributes = {{.type = pw::attribute::normals,
                        .data = std::vector{pw::vector{1.2f, 2.4f, 4.8f}}}}};

    for (auto i : geom.indices) {
        std::print("vertex[{}]({})\n", i, geom.vertices[i]);
    }

    auto aabb = pw::aabb<float, 3>::make_from_indexed_vertices(geom.vertices,
                                                               geom.indices);

    std::print("aabb.max = ({}) aabb.min=({})\n", aabb.max, aabb.min);
}
