#ifndef PW_SCENE_TRANSFORM_HPP
#define PW_SCENE_TRANSFORM_HPP

#include <pw/core/matrix.hpp>
#include <pw/core/quaternion.hpp>
#include <pw/core/vector.hpp>

namespace pw {

struct transform final {

    void set_global(const matrix4x4f& global) {
        matrix4x4f diff = _global.inverse() * global;
        _local          = _local * diff;

        _global = global;
    }

    inline transform& translate(const float& x, const float& y,
                                const float& z) {
        _local[0][3] += x;
        _local[1][3] += y;
        _local[2][3] += z;
        return *this;
    }

    inline transform& set_translation(const float& x, const float& y,
                                      const float& z) {
        _local[0][3] = x;
        _local[1][3] = y;
        _local[2][3] = z;
        return *this;
    }

    inline transform& rotate(const quaternionf& q) {
        _local = _local * q.to_matrix().unproject(1);
        return *this;
    }

    inline transform& set_rotation(const quaternionf& q) {
        _local = q.to_matrix().unproject(1);
        return *this;
    }

    inline transform& scale(const float& sx, const float& sy, const float& sz) {
        _local[0][0] *= sx;
        _local[1][1] *= sy;
        _local[2][2] *= sz;
        return *this;
    }

    inline transform& set_scale(const float& sx, const float& sy,
                                const float& sz) {
        _local[0][0] = sx;
        _local[1][1] = sy;
        _local[2][2] = sz;
        return *this;
    }

    inline transform& scale(const float& uniform_scale) {
        return scale(uniform_scale, uniform_scale, uniform_scale);
    }

    matrix4x4f _local  = matrix4x4f::identity();
    matrix4x4f _global = matrix4x4f::identity();
};

} // namespace pw

#endif
