#ifndef PW_SYSTEM_WINDOW_HPP
#define PW_SYSTEM_WINDOW_HPP

#include <cstdint>
#include <pw/core/globals.hpp>
#include <pw/core/point.hpp>
#include <pw/core/size.hpp>

#include <functional>

namespace pw {

struct window final {

    using on_update_t = std::function<void(window&)>;
    using on_resize_t = std::function<void(window&)>;

    window();
    ~window();

    bool update();

    void set_title(const std::string& title);

    void set_size(const size<int32_t>& s);
    pw::size<int32_t> size() const;
    pw::size<int32_t> client_size() const;

    void set_position(const point<int32_t>& p);
    point<int32_t> position() const;

    typedef void drop_callback;

    bool fullscreen() const;
    void set_fullscreen(bool use_fullscreen);

    void set_on_update(on_update_t f) { _on_update = f; }
    void set_on_resize(on_resize_t f) { _on_resize = f; }

    bool visible() const;
    void set_visible(bool is_visible);

  protected:
    struct impl;
    std::unique_ptr<impl> _impl;

    on_update_t _on_update;
    on_resize_t _on_resize;
};

} // namespace pw

#endif
